/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.user.copy.internal;

import static com.openexchange.java.Autoboxing.I;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import com.openexchange.user.copy.ObjectMapping;


/**
 * {@link IntegerToObjectMapping}
 *
 * @author <a href="mailto:steffen.templin@open-xchange.com">Steffen Templin</a>
 */
public abstract class IntegerToObjectMapping<T> implements ObjectMapping<T> {

    private final Map<Integer, Integer> idMapping;

    private final Map<Integer, T> sourceMapping;

    private final Map<Integer, T> destinationMapping;


    public IntegerToObjectMapping() {
        super();
        idMapping = new HashMap<Integer, Integer>();
        sourceMapping = new HashMap<Integer, T>();
        destinationMapping = new HashMap<Integer, T>();
    }

    /**
     * @see com.openexchange.user.copy.ObjectMapping#getSource(int)
     */
    @Override
    public T getSource(final int id) {
        return sourceMapping.get(I(id));
    }

    /**
     * @see com.openexchange.user.copy.ObjectMapping#getSourceKeys()
     */
    @Override
    public Set<Integer> getSourceKeys() {
        final Set<Integer> keySet = new HashSet<Integer>(idMapping.keySet());

        return keySet;
    }

    /**
     * @see com.openexchange.user.copy.ObjectMapping#getDestination(java.lang.Object)
     */
    @Override
    public abstract T getDestination(T source);

    protected T getDestinationById(final Integer id) {
        final Integer dstObjId = idMapping.get(id);
        if (dstObjId != null) {
            return destinationMapping.get(dstObjId);
        }

        return null;
    }

    public void addMapping(final Integer sourceId, final T source, final Integer destinationId, final T destination) {
        idMapping.put(sourceId, destinationId);
        sourceMapping.put(sourceId, source);
        destinationMapping.put(destinationId, destination);
    }

}
